@@ -51,7 +51,10 @@ req_auth_aws_v4 <- function(req,
       aws_session_token = aws_session_token,
       aws_service = aws_service,
       aws_region = aws_region
-    )
+    ),
+    # auth_aws_sign doesn't have anything to cache, but still need to provide
+    # a cache argument because the req_auth_sign machinery expects it
+    cache = cache_noop()
   )
 }
 
@@ -61,7 +64,7 @@ auth_aws_sign <- function(req,
                           aws_session_token = NULL,
                           aws_service = NULL,
                           aws_region = NULL,
-                          reauth = FALSE) {
+                          cache) {
 
   current_time <- Sys.time()
 
@@ -129,7 +132,7 @@ aws_v4_signature <- function(method,
     CanonicalQueryString <- ""
   } else {
     sorted_query <- url$query[order(names(url$query))]
-    CanonicalQueryString <- url_query_build(CanonicalQueryString)
+    CanonicalQueryString <- url_query_build(sorted_query)
   }
 
   headers$host <- url$hostname
