@@ -42,10 +42,11 @@ test_that("can determine if incomplete data is complete", {
   })
 
   con <- req %>% req_perform_connection(blocking = TRUE)
-  expect_equal(resp_stream_sse(con, 10), list(type = "message", data = "1", id = character()))
+  withr::defer(close(con))
+
+  expect_equal(resp_stream_sse(con, 10), list(type = "message", data = "1", id = ""))
   expect_snapshot(expect_equal(resp_stream_sse(con), NULL))
   expect_true(resp_stream_is_complete(con))
-  close(con)
 })
 
 test_that("can't read from a closed connection", {
@@ -60,13 +61,14 @@ test_that("can't read from a closed connection", {
 })
 
 test_that("can join lines across multiple reads", {
+  sync <- sync_req("join")
   req <- local_app_request(function(req, res) {
+    sync <- req$app$locals$sync_rep("join")
+
     res$send_chunk("This is a ")
-    Sys.sleep(0.2)
-    res$send_chunk("complete sentence.\n")
+    sync(res$send_chunk("complete sentence.\n"))
   })
 
-  # Non-blocking returns NULL until data is ready
   resp1 <- req_perform_connection(req, blocking = FALSE)
   withr::defer(close(resp1))
 
@@ -74,70 +76,71 @@ test_that("can join lines across multiple reads", {
   expect_equal(out, character())
   expect_equal(resp1$cache$push_back, charToRaw("This is a "))
 
-  while (length(out) == 0) {
-    Sys.sleep(0.1)
-    out <- resp_stream_lines(resp1)
-  }
+  out <- resp_stream_lines(resp1)
+  expect_equal(out, character())
+
+  sync()
+  out <- resp_stream_lines(resp1)
   expect_equal(out, "This is a complete sentence.")
 })
 
 test_that("handles line endings of multiple kinds", {
+  sync <- sync_req("endings")
+  req <- local_app_request(function(req, res) {
+    sync <- req$app$locals$sync_rep("endings")
+
+    res$set_header("Content-Type", "text/plain; charset=Shift_JIS")
+    res$send_chunk(as.raw(c(0x82, 0xA0, 0x0A)))
+    sync(res$send_chunk("crlf\r\n"))
+    sync(res$send_chunk("lf\n"))
+    sync(res$send_chunk("cr\r"))
+    sync(res$send_chunk("half line/"))
+    sync(res$send_chunk("other half\n"))
+    sync(res$send_chunk("broken crlf\r"))
+    sync(res$send_chunk("\nanother line\n"))
+    sync(res$send_chunk("eof without line ending"))
+  })
+
+  resp1 <- req_perform_connection(req, blocking = FALSE)
+  withr::defer(close(resp1))
+
+  expected_values <- list(
+    "\u3042", "crlf", "lf", "cr", character(0), "half line/other half", "broken crlf", "another line"
+  )
+
+  for (expected in expected_values) {
+    rlang::inject(expect_equal(resp_stream_lines(resp1), !!expected))
+    sync()
+  }
+  expect_warning(out <- resp_stream_lines(resp1), "incomplete final line")
+  expect_equal(out, "eof without line ending")
+  expect_equal(resp_stream_lines(resp1), character(0))
+
+  # Same test, but now, blocking (and without sync)
   req <- local_app_request(function(req, res) {
     res$set_header("Content-Type", "text/plain; charset=Shift_JIS")
     res$send_chunk(as.raw(c(0x82, 0xA0, 0x0A)))
-    Sys.sleep(0.1)
     res$send_chunk("crlf\r\n")
-    Sys.sleep(0.1)
     res$send_chunk("lf\n")
-    Sys.sleep(0.1)
     res$send_chunk("cr\r")
-    Sys.sleep(0.1)
     res$send_chunk("half line/")
-    Sys.sleep(0.1)
     res$send_chunk("other half\n")
-    Sys.sleep(0.1)
     res$send_chunk("broken crlf\r")
-    Sys.sleep(0.1)
     res$send_chunk("\nanother line\n")
-    Sys.sleep(0.1)
     res$send_chunk("eof without line ending")
   })
-
-  resp1 <- req_perform_connection(req, blocking = TRUE)
-  withr::defer(close(resp1))
-
-  for (expected in c("\u3042", "crlf", "lf", "cr", "half line/other half", "broken crlf", "another line")) {
-    rlang::inject(expect_equal(resp_stream_lines(resp1), !!expected))
-  }
-  expect_warning(
-    expect_equal(resp_stream_lines(resp1), "eof without line ending"),
-    "incomplete final line"
-  )
-  expect_identical(resp_stream_lines(resp1), character(0))
-
-  # Same test, but now, non-blocking
-  resp2 <- req_perform_connection(req, blocking = FALSE)
+  resp2 <- req_perform_connection(req, blocking = TRUE)
   withr::defer(close(resp2))
 
-  for (expected in c("\u3042", "crlf", "lf", "cr", "half line/other half", "broken crlf", "another line")) {
-    repeat {
-      out <- resp_stream_lines(resp2)
-      if (length(out) > 0) {
-        rlang::inject(expect_equal(out, !!expected))
-        break
-      }
-    }
-  }
-  expect_warning(
-    repeat {
-      out <- resp_stream_lines(resp2)
-      if (length(out) > 0) {
-        expect_equal(out, "eof without line ending")
-        break
-      }
-    },
-    "incomplete final line"
+  expected_values <- c(
+    "\u3042", "crlf", "lf", "cr", "half line/other half", "broken crlf", "another line"
   )
+
+  for (expected in expected_values) {
+    rlang::inject(expect_equal(resp_stream_lines(resp2), !!expected))
+  }
+  expect_warning(out <- resp_stream_lines(resp2), "incomplete final line")
+  expect_equal(out, "eof without line ending")
 })
 
 test_that("streams the specified number of lines", {
@@ -147,35 +150,16 @@ test_that("streams the specified number of lines", {
 
   resp1 <- req_perform_connection(req, blocking = TRUE)
   withr::defer(close(resp1))
-  expect_equal(
-    resp_stream_lines(resp1, 3),
-    c("a", "b", "c")
-  )
-  expect_equal(
-    resp_stream_lines(resp1, 3),
-    c("d", "e")
-  )
-  expect_equal(
-    resp_stream_lines(resp1, 3),
-    character()
-  )
+  expect_equal(resp_stream_lines(resp1, 3), c("a", "b", "c"))
+  expect_equal(resp_stream_lines(resp1, 3), c("d", "e"))
+  expect_equal(resp_stream_lines(resp1, 3), character())
 
   resp2 <- req_perform_connection(req, blocking = FALSE)
   withr::defer(close(resp2))
-  Sys.sleep(0.2)
-  expect_equal(
-    resp_stream_lines(resp2, 3),
-    c("a", "b", "c")
-  )
-  expect_equal(
-    resp_stream_lines(resp2, 3),
-    c("d", "e")
-  )
-  expect_equal(
-    resp_stream_lines(resp2, 3),
-    character()
-  )
-})
+  expect_equal(resp_stream_lines(resp2, 3), c("a", "b", "c"))
+  expect_equal(resp_stream_lines(resp2, 3), c("d", "e"))
+  expect_equal(resp_stream_lines(resp2, 3), character())
+ })
 
 test_that("can feed sse events one at a time", {
   req <- local_app_request(function(req, res) {
@@ -188,25 +172,40 @@ test_that("can feed sse events one at a time", {
 
   expect_equal(
     resp_stream_sse(resp),
-    list(type = "message", data = "1", id = character())
+    list(type = "message", data = "1", id = "")
   )
   expect_equal(
     resp_stream_sse(resp),
-    list(type = "message", data = "2", id = character())
+    list(type = "message", data = "2", id = "")
   )
   resp_stream_sse(resp)
 
   expect_equal(resp_stream_sse(resp), NULL)
 })
 
-test_that("can join sse events across multiple reads", {
+test_that("ignores events with no data", {
   req <- local_app_request(function(req, res) {
+    res$send_chunk(": comment\n\n")
+    res$send_chunk("data: 1\n\n")
+  })
+  resp <- req_perform_connection(req)
+  withr::defer(close(resp))
+
+  expect_equal(
+    resp_stream_sse(resp),
+    list(type = "message", data = "1", id = "")
+  )
+})
+
+test_that("can join sse events across multiple reads", {
+  sync <- sync_req("sse")
+  req <- local_app_request(function(req, res) {
+    sync <- req$app$locals$sync_rep("sse")
+
     res$send_chunk("data: 1\n")
-    Sys.sleep(0.2)
-    res$send_chunk("data")
-    Sys.sleep(0.2)
+    sync(res$send_chunk("data"))
     res$send_chunk(": 2\n")
-    res$send_chunk("\ndata: 3\n\n")
+    sync(res$send_chunk("\ndata: 3\n\n"))
   })
 
   # Non-blocking returns NULL until data is ready
@@ -217,21 +216,30 @@ test_that("can join sse events across multiple reads", {
   expect_equal(out, NULL)
   expect_equal(resp1$cache$push_back, charToRaw("data: 1\n"))
 
-  while (is.null(out)) {
-    Sys.sleep(0.1)
-    out <- resp_stream_sse(resp1)
-  }
-  expect_equal(out, list(type = "message", data = c("1", "2"), id = character()))
-  expect_equal(resp1$cache$push_back, charToRaw("data: 3\n\n"))
+  sync()
   out <- resp_stream_sse(resp1)
-  expect_equal(out, list(type = "message", data = "3", id = character()))
+  expect_equal(out, NULL)
 
-  # Blocking waits for a complete event
+  sync()
+  out <- resp_stream_sse(resp1)
+  expect_equal(out, list(type = "message", data = "1\n2", id = ""))
+  expect_equal(resp1$cache$push_back, charToRaw("data: 3\n\n"))
+
+  out <- resp_stream_sse(resp1)
+  expect_equal(out, list(type = "message", data = "3", id = ""))
+
+  # # Blocking waits for a complete event
+  req <- local_app_request(function(req, res) {
+    res$send_chunk("data: 1\n")
+    res$send_chunk("data")
+    res$send_chunk(": 2\n")
+    res$send_chunk("\ndata: 3\n\n")
+  })
   resp2 <- req_perform_connection(req)
   withr::defer(close(resp2))
 
   out <- resp_stream_sse(resp2)
-  expect_equal(out, list(type = "message", data = c("1", "2"), id = character()))
+  expect_equal(out, list(type = "message", data = "1\n2", id = ""))
 })
 
 test_that("sse always interprets data as UTF-8", {
@@ -239,23 +247,18 @@ test_that("sse always interprets data as UTF-8", {
     res$send_chunk("data: \xE3\x81\x82\r\n\r\n")
   })
 
-  withr::with_locale(c(LC_CTYPE = "C"), {
-    # Non-blocking returns NULL until data is ready
-    resp1 <- req_perform_connection(req, blocking = FALSE)
-    withr::defer(close(resp1))
+  withr::local_locale(LC_CTYPE = "C")
+  # Non-blocking returns NULL until data is ready
+  resp1 <- req_perform_connection(req, blocking = FALSE)
+  withr::defer(close(resp1))
 
-    out <- NULL
-    while (is.null(out)) {
-      Sys.sleep(0.1)
-      out <- resp_stream_sse(resp1)
-    }
+  out <- resp_stream_sse(resp1)
 
-    s <- "\xE3\x81\x82"
-    Encoding(s) <- "UTF-8"
-    expect_equal(out, list(type = "message", data = s, id = character()))
-    expect_equal(Encoding(out$data), "UTF-8")
-    expect_equal(resp1$cache$push_back, raw())
-  })
+  s <- "\xE3\x81\x82"
+  Encoding(s) <- "UTF-8"
+  expect_equal(out, list(type = "message", data = s, id = ""))
+  expect_equal(Encoding(out$data), "UTF-8")
+  expect_equal(resp1$cache$push_back, raw())
 })
 
 test_that("streaming size limits enforced", {
@@ -268,22 +271,15 @@ test_that("streaming size limits enforced", {
   resp1 <- req_perform_connection(req, blocking = FALSE)
   withr::defer(close(resp1))
   expect_error(
-    while (is.null(out)) {
-      Sys.sleep(0.1)
-      out <- resp_stream_sse(resp1, max_size = 999)
-    }
+    out <- resp_stream_sse(resp1, max_size = 999),
+    class = "httr2_streaming_error"
   )
 
   resp2 <- req_perform_connection(req, blocking = TRUE)
   withr::defer(close(resp2))
   expect_error(
-    out <- resp_stream_sse(resp2, max_size = 999)
-  )
-
-  resp3 <- req_perform_connection(req, blocking = TRUE)
-  withr::defer(close(resp3))
-  expect_error(
-    out <- resp_stream_lines(resp3, max_size = 999)
+    out <- resp_stream_sse(resp2, max_size = 999),
+    class = "httr2_streaming_error"
   )
 })
 
@@ -295,7 +291,7 @@ test_that("verbosity = 2 streams request bodies", {
 
   stream_all <- function(req, fun, ...) {
     con <- req_perform_connection(req, blocking = TRUE, verbosity = 2)
-    on.exit(close(con))
+    withr::defer(close(con))
     while (!resp_stream_is_complete(con)) {
       fun(con, ...)
     }
@@ -315,7 +311,7 @@ test_that("verbosity = 3 shows buffer info", {
     res$send_chunk("line 2\n")
   })
 
-  con <- req_perform_connection(req, blocking = TRUE, verbosity = 3)
+  expect_output(con <- req_perform_connection(req, blocking = TRUE, verbosity = 3))
   on.exit(close(con))
   expect_snapshot(
     {
@@ -323,10 +319,23 @@ test_that("verbosity = 3 shows buffer info", {
         resp_stream_lines(con, 1)
       }
     },
-    transform = function(lines) lines[!grepl("^(<-|->)", lines)]
+    transform = transform_verbose_response
   )
 })
 
+test_that("verbosity = 3 shows raw sse events", {
+  req <- local_app_request(function(req, res) {
+    res$send_chunk(": comment\n\n")
+    res$send_chunk("data: 1\n\n")
+  })
+
+  expect_output(resp <- req_perform_connection(req, verbosity = 3))
+  withr::defer(close(resp))
+  expect_snapshot(
+    . <- resp_stream_sse(resp),
+    transform = transform_verbose_response
+  )
+})
 
 test_that("has a working find_event_boundary", {
   boundary_test <- function(x, matched, remaining) {
@@ -368,3 +377,20 @@ test_that("has a working find_event_boundary", {
   expect_null(find_event_boundary(charToRaw("12")))
   expect_null(find_event_boundary(charToRaw("\r\n\r")))
 })
+
+# parse_event ----------------------------------------------------------------
+
+test_that("event with no data returns NULL", {
+  expect_null(parse_event(""))
+  expect_null(parse_event(":comment"))
+  expect_null(parse_event("id: 1"))
+
+  expect_equal(parse_event("data: ")$data, "")
+  expect_equal(parse_event("data")$data, "")
+})
+
+test_that("examples from spec work", {
+  event <- parse_event("data: YHOO\ndata: +2\ndata: 10")
+  expect_equal(event$type, "message")
+  expect_equal(event$data, "YHOO\n+2\n10")
+})
